package com.example.catchTheLetters.service;

import com.example.catchTheLetters.entity.GameRoom;
import com.example.catchTheLetters.entity.PlayerInput;
import org.springframework.web.socket.WebSocketSession;

import java.util.List;

/**
 * 房间服务
 *
 * @author spyn
 */
public interface RoomService {
    /**
     * 添加玩家
     * @param roomId 房间号
     * @param session WebSocket 会话
     * @param token 玩家 token
     */
    void addPlayer(long roomId, WebSocketSession session, String token);

    /**
     * 移除玩家
     * @param roomId 房间号
     * @param session WebSocket 会话
     */
    void removePlayer(long roomId, WebSocketSession session);

    /**
     * 踢出玩家
     * @param roomId 房间号
     * @param session 房主的 WebSocket 会话
     * @param playerID 要踢出的玩家 ID
     */
    void kickPlayer(long roomId, WebSocketSession session, String playerID);

    /**
     * 开始游戏
     * @param roomId 房间号
     * @param session WebSocket 会话
     */
    void startGame(long roomId, WebSocketSession session);

    /**
     * 取消开始游戏
     * @param roomId 房间号
     * @param session WebSocket 会话
     */
    void cancelStartGame(long roomId, WebSocketSession session);

    /**
     * 处理玩家输入
     * @param roomId 房间号
     * @param session WebSocket 会话
     * @param input 玩家输入
     */
    void handleInput(long roomId, WebSocketSession session, PlayerInput input);

    /**
     * 创建房间
     * @param session WebSocket 会话
     * @param token 玩家 token
     */
    void createRoom(WebSocketSession session, String token);

    /**
     * 移除房间
     * @param roomId 房间号
     */
    void removeRoom(long roomId);

    /**
     * 邀请玩家
     * @param roomId 房间号
     * @param session WebSocket 会话
     * @param playerID 被邀请的玩家ID
     */
    void invitePlayer(long roomId, WebSocketSession session, String playerID);

    /**
     * 连接ws
     * @param session WebSocket 会话
     * @param token 玩家 token
     */
    void connect(WebSocketSession session, String token);

    /**
     * 断开ws
     * @param token 玩家 token
     */
    void disconnect(String token);

    /**
     * 发送消息
     * @param session WebSocket 会话
     * @param message 消息
     * @param <T> 消息类型
     */
    <T> void sendMessage(WebSocketSession session, T message);

    /**
     * 玩家获得了字母
     * @param roomId 房间号
     * @param session WebSocket 会话
     * @param letterId 字母ID
     */
    void getLetter(long roomId, WebSocketSession session, long letterId);

    List<GameRoom> getAllRoom();
}
